
Sys.setenv(LANG = "en")
# 

library(dlnm) ; library(mixmeta) ; library(splines) ; library(tsModel)
library(mgcv) 
library(data.table); library(reshape2); library(ggplot2)

load("C:/Users/ed19w187/Desktop/PhD/PhD/Publication_GCD/FINAL_CODE/review_comments/temperature_data_CH.Rdata")

#####
#QAIC function
QAIC <- function(model){
  phi <- summary(model)$dispersion
  loglik <-sum(dpois(model$y, model$fitted.values, log=TRUE))
  return(-2*loglik+2*summary(model)$df[3]*phi)
}


namesreg_full <- c("Zrich", "Bern","Luzern", "Uri", "Schwyz", "Obwalden", "Nidwalden","Glarus", "Zug","Fribourg","Solothurn","Basel-Stadt", "Basel-Landschaft","Schaffhausen","Appenzell Ausserhoden", "Appenzell Innerhoden","St. Gallen", "Grabunden", "Aargau", "Thurgau", "Ticino", "Vaud", "Valais","Neuchtel", "Genve", "Jura")
namesreg <- namesreg_full

## DEFINITON OF THE PARAMETERS
# SPECIFICATION OF THE EXPOSURE FUNCTION
varfun <- "bs"
vardegree <- 2
varper <- c(10,75,90)

# SPECIFICATION OF THE LAG FUNCTION
lag <- 21
lagnk <- 3

# DEGREE OF FREEDOM FOR SEASONALITY
dfseas <- 8

# DEFINE THE OUTCOME
out <- "deaths"

# MODEL FORMULA
formula <- deaths ~ cb + dow + ns(date,df=round(dfseas*length(date)/365.25))


## 1ST STAGE
# CREATE EMPTY OBJECTS TO STORE RESULTS
ncoef <- length(varper) + ifelse(varfun=="bs",vardegree,1)

coefallinsitu <- matrix(NA,length(namesreg),ncoef,dimnames=list(namesreg))
vcovallinsitu <- vector("list",length(namesreg))

coefallTabsd <- matrix(NA,length(namesreg),ncoef,dimnames=list(namesreg))
vcovallTabsd <- vector("list",length(namesreg))

coefallTabsd_uw <- matrix(NA,length(namesreg),ncoef,dimnames=list(namesreg))
vcovallTabsd_uw <- vector("list",length(namesreg))

coefallERA5 <- matrix(NA,length(namesreg),ncoef,dimnames=list(namesreg))
vcovallERA5 <- vector("list",length(namesreg))

coefallERA5_uw <- matrix(NA,length(namesreg),ncoef,dimnames=list(namesreg))
vcovallERA5_uw <- vector("list",length(namesreg))

qaicres <- matrix(NA, ncol=5, nrow=26)
rownames(qaicres) <- namesreg
names(vcovallinsitu) <- names(vcovallTabsd) <- names(vcovallTabsd_uw) <- names(vcovallERA5) <- names(vcovallERA5_uw) <- namesreg

redallinsitu <- redallTabsd <- redallTabsd_uw <- redallERA5 <- redallERA5_uw <- list()

# TABLE HEAT
tab_heat <- matrix(NA, nrow=26, ncol=5)
rownames(tab_heat) <- namesreg


tab_heat_insitu <- tab_heat_Tabsd  <- tab_heat_Tabsd_uw <- tab_heat_ERA5 <- tab_heat_ERA5_uw <- matrix(NA, nrow=length(fulldataset), ncol=3)
rownames(tab_heat_insitu) <- rownames(tab_heat_Tabsd ) <- rownames(tab_heat_Tabsd_uw) <- rownames(tab_heat_ERA5) <- rownames(tab_heat_ERA5_uw) <- rep(names(fulldataset))
tab_heat_insitu[,1] 

# TABLE COLD
tab_cold <- matrix(NA, nrow=26, ncol=5)
rownames(tab_cold) <- namesreg


tab_cold_insitu <- tab_cold_Tabsd <- tab_cold_Tabsd_uw <- tab_cold_ERA5 <- tab_cold_ERA5_uw <- matrix(NA, nrow=length(fulldataset), ncol=3)
rownames(tab_cold_insitu) <- rownames(tab_cold_Tabsd) <- rownames(tab_cold_Tabsd_uw) <- rownames(tab_cold_ERA5) <- rownames(tab_cold_ERA5_uw) <- rep(names(fulldataset))



colnames(tab_heat) <- colnames(tab_cold) <- c("insitu", "Tabsd", "Tabsd_unw", "era5", "era5_unw")

# OBJECTS FOR THE ESTIMATIONS OF ATTRIBUTABLE FRACTIONS
# LOAD FUNCTION
source("C:/Users/ed19w187/Desktop/PhD/PhD/Mortality/attrdl.r")

# CREATE THE VECTORS TO STORE THE TOTAL MORTALITY (ACCOUNTING FOR MISSING)
totdeathall <- rep(NA,length(namesreg))
names(totdeathall) <- namesreg

# CREATE THE MATRIX TO STORE THE ATTRIBUTABLE DEATHS USING THE CITYBLUPS
matsim_insitu <- matsim_Tabsd <- matsim_Tabsd_uw <- matsim_era5  <- matsim_era5_uw <- matrix(NA,length(namesreg),5,
                                                                                             dimnames=list(namesreg,c("cold","m25", "m75", "heat","total")))

# NUMBER OF SIMULATION RUNS FOR COMPUTING EMPIRICAL CI
nsim <- 1000

# CREATE THE ARRAY TO STORE THE CI OF ATTRIBUTABLE DEATHS USING THE CITYBLUPS
arraysim_insitu <- arraysim_Tabsd <- arraysim_Tabsd_uw <- arraysim_era5<- arraysim_era5_uw <- array(NA,dim=c(length(namesreg),5,nsim),dimnames=list(namesreg,
                                                                                                                                                    c("cold","m25", "m75", "heat","total")))

# MODEL FORMULA
formula <- deaths ~ cb + dow + ns(date,df=round(dfseas*length(date)/365.25))


#mmt_list <- matrix(NA, nrow=26, ncol=5)

# DEFINITION SEQ OF PERCENTILES
predper <- c(seq(0,1,0.1),2:98,seq(99,100,0.1))

# RUN LOOP ACROSS REGIONS
for(i in seq(length(fulldataset))) {
  
  # PRINT
  cat(i,"")
  
  # EXTRACT THE DATA FOR REGION i
  data <- fulldataset[[i]]
  
  
  #### - Insitu  ####
  # DEFINE THE CROSSBASIs
  argvar <- list(fun=varfun, degree=vardegree,
                 knots=quantile(data$monitor,varper/100,na.rm=T), 
                 Bound=range(data$monitor,na.rm=T))
  
  
  arglag <- list(fun="ns",knots=logknots(lag,lagnk))
  cb <- crossbasis(data$monitor,lag=lag,argvar=argvar,arglag=arglag)
  
  # RUN THE MODEL AND OBTAIN PREDICTIONS
  model <- glm(formula,data,family=quasipoisson,na.action="na.exclude")
  
  # REDUCTION TO OVERALL CUMULATIVE
  redtemp <- crossreduce(cb,model,cen=mean(data$monitor,na.rm=T))
  pred <- crosspred(cb,model,cen=mean(data$monitor,na.rm=T))
  redtemp$predvar2 <- quantile(data$monitor, 1:99/100, na.rm=T)
  #mmt <-  redtemp$predvar2[40:99][which.min(redtemp$fit)[[1]]]
  mmt <- mmt_list[i,1] 
  
  
  
  
  redallinsitu[[i]] <- crossreduce(cb,model,cen=mmt)
  plot(redallinsitu[[i]])
  
  coefallinsitu[i,] <- coef(redallinsitu[[i]])
  vcovallinsitu[[i]] <- vcov(redallinsitu[[i]])
  qaicres[i,1] <- QAIC(model)
  
  cutheatinsitu<- quantile(data$monitor,0.99, na.rm=T)
  cutcoldinsitu <- quantile(data$monitor,0.01, na.rm=T)
  cutinsitu <- crossreduce(cb,model, at=c(cutheatinsitu,cutcoldinsitu), cen=mmt)
  
  
  tab_heat[i,1] <- paste0(formatC(cutinsitu$RRfit[2],2, format="f"), sep=" (", 
                          formatC(cutinsitu$RRlow[2],2, format="f"), sep=" - ",  
                          formatC(cutinsitu$RRhigh[2],2, format="f"), sep=")")
  
  tab_cold[i,1] <- paste0(formatC(cutinsitu$RRfit[1],2, format="f"), sep=" (", 
                          formatC(cutinsitu$RRlow[1],2, format="f"), sep=" - ",  
                          formatC(cutinsitu$RRhigh[1],2, format="f"), sep=")")
  
  tab_heat_insitu[i,1:3] <- cbind(cutinsitu$RRfit[2], cutinsitu$RRlow[2],cutinsitu$RRhigh[2])
  tab_cold_insitu[i,1:3] <- cbind(cutinsitu$RRfit[1], cutinsitu$RRlow[1],cutinsitu$RRhigh[1])
  
  # ESTIMATION OF THE ATTRIBUTABLE FRACTIONS
  matsim_insitu[i,"heat"] <- attrdl(data$monitor,cb,data$deaths,coef=coefallinsitu[i,],
                                    vcov=vcovallinsitu[[i]],type="an",dir="forw",
                                    range=c(quantile(data$monitor, 0.90, na.rm=T),100), cen=mmt)
  
  matsim_insitu[i,"m75"] <- attrdl(data$monitor,cb,data$deaths,coef=coefallinsitu[i,],
                                   vcov=vcovallinsitu[[i]],type="an",dir="forw",
                                   range=c(quantile(data$monitor, 0.75, na.rm=T),100), cen=mmt)
  
  matsim_insitu[i,"cold"] <- attrdl(data$monitor,cb,data$deaths,coef=coefallinsitu[i,],
                                    vcov=vcovallinsitu[[i]],type="an",dir="forw",
                                    range=c(-100,quantile(data$monitor, 0.1, na.rm=T)), cen=mmt)
  
  matsim_insitu[i,"m25"] <- attrdl(data$monitor,cb,data$deaths,coef=coefallinsitu[i,],
                                   vcov=vcovallinsitu[[i]],type="an",dir="forw",
                                   range=c(-100,quantile(data$monitor, 0.25, na.rm=T)), cen=mmt)
  
  matsim_insitu[i,"total"] <- attrdl(data$monitor,cb,data$deaths,coef=coefallinsitu[i,],
                                     vcov=vcovallinsitu[[i]],type="an",dir="forw", cen=mmt)
  
  # COMPUTE EMPIRICAL OCCURRENCES OF THE ATTRIBUTABLE DEATHS
  # USED TO DERIVE CONFIDENCE INTERVALS
  arraysim_insitu[i,"heat",] <- attrdl(data$monitor,cb,data$deaths,coef=coefallinsitu[i,],
                                       vcov=vcovallinsitu[[i]],type="an",dir="forw", sim=T,nsim=nsim,
                                       range=c(quantile(data$monitor, 0.90, na.rm=T),100), cen=mmt)
  
  arraysim_insitu[i,"m75",] <- attrdl(data$monitor,cb,data$deaths,coef=coefallinsitu[i,],
                                      vcov=vcovallinsitu[[i]],type="an",dir="forw", sim=T,nsim=nsim,
                                      range=c(quantile(data$monitor, 0.75, na.rm=T),100), cen=mmt)
  
  arraysim_insitu[i,"cold",] <- attrdl(data$monitor,cb,data$deaths,coef=coefallinsitu[i,],
                                       vcov=vcovallinsitu[[i]],type="an",dir="forw", sim=T,nsim=nsim,
                                       range=c(-100,quantile(data$monitor, 0.1, na.rm=T)), cen=mmt)
  
  arraysim_insitu[i,"m25",] <- attrdl(data$monitor,cb,data$deaths,coef=coefallinsitu[i,],
                                      vcov=vcovallinsitu[[i]],type="an",dir="forw", sim=T,nsim=nsim,
                                      range=c(-100,quantile(data$monitor, 0.25, na.rm=T)), cen=mmt)
  
  arraysim_insitu[i,"total",] <- attrdl(data$monitor,cb,data$deaths,coef=coefallinsitu[i,],
                                        vcov=vcovallinsitu[[i]],type="an",dir="forw", sim=T, nsim=nsim,cen=mmt)
  
  
  
  
  #### - TABSD ####
  # DEFINE THE CROSSBASIs
  argvar <- list(fun=varfun, degree=vardegree,
                 knots=quantile(data$Tabsd,varper/100,na.rm=T), 
                 Bound=range(data$Tabsd,na.rm=T))
  
  
  arglag <- list(fun="ns",knots=logknots(lag,lagnk))
  cb <- crossbasis(data$Tabsd,lag=lag,argvar=argvar,arglag=arglag)
  
  # RUN THE MODEL AND OBTAIN PREDICTIONS
  model <- glm(formula,data,family=quasipoisson,na.action="na.exclude")
  
  # REDUCTION TO OVERALL CUMULATIVE
  redtemp <- crossreduce(cb,model,cen=mean(data$Tabsd,na.rm=T))
  pred <- crosspred(cb,model,cen=mean(data$Tabsd,na.rm=T))
  redtemp$predvar2 <- quantile(data$Tabsd, 1:99/100)
  #mmt <-  redtemp$predvar2[40:99][which.min(redtemp$fit)[[1]]]
  mmt <- mmt_list[i,2] 
  
  redallTabsd[[i]] <- crossreduce(cb,model,cen=mmt)
  plot(redallTabsd[[i]])
  coefallTabsd[i,] <- coef(redallTabsd[[i]])
  vcovallTabsd[[i]] <- vcov(redallTabsd[[i]])
  qaicres[i,2] <- QAIC(model)
  
  cutheatTabsd<- quantile(data$Tabsd,0.99, na.rm=T)
  cutcoldTabsd <- quantile(data$Tabsd,0.01, na.rm=T)
  cutTabsd<- crossreduce(cb,model, at=c(cutheatTabsd,cutcoldTabsd), cen=mmt)
  
  
  tab_heat[i,2] <- paste0(formatC(cutTabsd$RRfit[2],2, format="f"), sep=" (", 
                          formatC(cutTabsd$RRlow[2],2, format="f"), sep=" - ",  
                          formatC(cutTabsd$RRhigh[2],2, format="f"), sep=")")
  
  tab_cold[i,2] <- paste0(formatC(cutTabsd$RRfit[1],2, format="f"), sep=" (", 
                          formatC(cutTabsd$RRlow[1],2, format="f"), sep=" - ",  
                          formatC(cutTabsd$RRhigh[1],2, format="f"), sep=")")
  
  tab_heat_Tabsd[i,1:3] <- cbind(cutTabsd$RRfit[2], cutTabsd$RRlow[2],cutTabsd$RRhigh[2])
  tab_cold_Tabsd[i,1:3] <- cbind(cutTabsd$RRfit[1], cutTabsd$RRlow[1],cutTabsd$RRhigh[1])
  
  # ESTIMATION OF THE ATTRIBUTABLE FRACTIONS
  matsim_Tabsd[i,"heat"] <- attrdl(data$Tabsd,cb,data$deaths,coef=coefallTabsd[i,],
                                   vcov=vcovallTabsd[[i]],type="an",dir="forw",
                                   range=c(quantile(data$Tabsd, 0.90, na.rm=T),100), cen=mmt)
  
  matsim_Tabsd[i,"m75"] <- attrdl(data$Tabsd,cb,data$deaths,coef=coefallTabsd[i,],
                                  vcov=vcovallTabsd[[i]],type="an",dir="forw",
                                  range=c(quantile(data$Tabsd, 0.75, na.rm=T),100), cen=mmt)
  
  matsim_Tabsd[i,"cold"] <- attrdl(data$Tabsd,cb,data$deaths,coef=coefallTabsd[i,],
                                   vcov=vcovallTabsd[[i]],type="an",dir="forw",
                                   range=c(-100,quantile(data$Tabsd, 0.1, na.rm=T)), cen=mmt)
  
  matsim_Tabsd[i,"m25"] <- attrdl(data$Tabsd,cb,data$deaths,coef=coefallTabsd[i,],
                                  vcov=vcovallTabsd[[i]],type="an",dir="forw",
                                  range=c(-100,quantile(data$Tabsd, 0.25, na.rm=T)), cen=mmt)
  
  matsim_Tabsd[i,"total"] <- attrdl(data$Tabsd,cb,data$deaths,coef=coefallTabsd[i,],
                                    vcov=vcovallTabsd[[i]],type="an",dir="forw", cen=mmt)
  
  # COMPUTE EMPIRICAL OCCURRENCES OF THE ATTRIBUTABLE DEATHS
  # USED TO DERIVE CONFIDENCE INTERVALS
  arraysim_Tabsd[i,"heat",] <- attrdl(data$Tabsd,cb,data$deaths,coef=coefallTabsd[i,],
                                      vcov=vcovallTabsd[[i]],type="an",dir="forw", sim=T,nsim=nsim,
                                      range=c(quantile(data$Tabsd, 0.90, na.rm=T),100), cen=mmt)
  
  arraysim_Tabsd[i,"m75",] <- attrdl(data$Tabsd,cb,data$deaths,coef=coefallTabsd[i,],
                                     vcov=vcovallTabsd[[i]],type="an",dir="forw", sim=T,nsim=nsim,
                                     range=c(quantile(data$Tabsd, 0.75, na.rm=T),100), cen=mmt)
  
  arraysim_Tabsd[i,"cold",] <- attrdl(data$Tabsd,cb,data$deaths,coef=coefallTabsd[i,],
                                      vcov=vcovallTabsd[[i]],type="an",dir="forw", sim=T,nsim=nsim,
                                      range=c(-100,quantile(data$Tabsd, 0.1, na.rm=T)), cen=mmt)
  
  arraysim_Tabsd[i,"m25",] <- attrdl(data$Tabsd,cb,data$deaths,coef=coefallTabsd[i,],
                                     vcov=vcovallTabsd[[i]],type="an",dir="forw", sim=T,nsim=nsim,
                                     range=c(-100,quantile(data$Tabsd, 0.25, na.rm=T)), cen=mmt)
  
  arraysim_Tabsd[i,"total",] <- attrdl(data$Tabsd,cb,data$deaths,coef=coefallTabsd[i,],
                                       vcov=vcovallTabsd[[i]],type="an",dir="forw", sim=T, nsim=nsim,cen=mmt)
  
  
  #### - UNWEIGHTED TABSD ####
  # DEFINE THE CROSSBASIs
  argvar <- list(fun=varfun, degree=vardegree,
                 knots=quantile(data$Tabsd_uw,varper/100,na.rm=T), 
                 Bound=range(data$Tabsd_uw,na.rm=T))
  #Bound=range(tmeanper,na.rm=T))
  
  arglag <- list(fun="ns",knots=logknots(lag,lagnk))
  cb <- crossbasis(data$Tabsd_uw,lag=lag,argvar=argvar,arglag=arglag)
  
  # RUN THE MODEL AND OBTAIN PREDICTIONS
  model <- glm(formula,data,family=quasipoisson,na.action="na.exclude")
  
  # REDUCTION TO OVERALL CUMULATIVE
  redtemp <- crossreduce(cb,model,cen=mean(data$Tabsd_uw,na.rm=T)) 
  pred <- crosspred(cb,model,cen=mean(data$Tabsd_uw,na.rm=T))
  redtemp$predvar2 <- quantile(data$Tabsd_uw, 1:99/100)
  #mmt <-  redtemp$predvar2[40:99][which.min(redtemp$fit)[[1]]]
  mmt <- mmt_list[i,3] 
  
  redallTabsd_uw[[i]] <- crossreduce(cb,model,cen=mmt)
  plot(redallTabsd_uw[[i]])
  
  coefallTabsd_uw[i,] <- coef(redallTabsd_uw[[i]])
  vcovallTabsd_uw[[i]] <- vcov(redallTabsd_uw[[i]])
  qaicres[i,3] <- QAIC(model)
  
  cutheatTabsd_uw<- quantile(data$Tabsd_uw,0.99, na.rm=T)
  cutcoldTabsd_uw <- quantile(data$Tabsd_uw,0.01, na.rm=T)
  cutTabsd_uw <- crossreduce(cb,model, at=c(cutheatTabsd_uw,cutcoldTabsd_uw), cen=mmt)
  
  
  tab_heat[i,3] <- paste0(formatC(cutTabsd_uw$RRfit[2],2, format="f"), sep=" (", 
                          formatC(cutTabsd_uw$RRlow[2],2, format="f"), sep=" - ",  
                          formatC(cutTabsd_uw$RRhigh[2],2, format="f"), sep=")")
  
  tab_cold[i,3] <- paste0(formatC(cutTabsd_uw$RRfit[1],2, format="f"), sep=" (", 
                          formatC(cutTabsd_uw$RRlow[1],2, format="f"), sep=" - ",  
                          formatC(cutTabsd_uw$RRhigh[1],2, format="f"), sep=")")
  
  tab_heat_Tabsd_uw[i,1:3] <- cbind(cutTabsd_uw$RRfit[2], cutTabsd_uw$RRlow[2],cutTabsd_uw$RRhigh[2])
  tab_cold_Tabsd_uw[i,1:3] <- cbind(cutTabsd_uw$RRfit[1], cutTabsd_uw$RRlow[1],cutTabsd_uw$RRhigh[1])
  
  # ESTIMATION OF THE ATTRIBUTABLE FRACTIONS
  matsim_Tabsd_uw[i,"heat"] <- attrdl(data$Tabsd_uw,cb,data$deaths,coef=coefallTabsd_uw[i,],
                                      vcov=vcovallTabsd_uw[[i]],type="an",dir="forw",
                                      range=c(quantile(data$Tabsd_uw, 0.9, na.rm=T),100), cen=mmt)
  
  matsim_Tabsd_uw[i,"m75"] <- attrdl(data$Tabsd_uw,cb,data$deaths,coef=coefallTabsd_uw[i,],
                                     vcov=vcovallTabsd_uw[[i]],type="an",dir="forw",
                                     range=c(quantile(data$Tabsd_uw, 0.75, na.rm=T),100), cen=mmt)
  
  matsim_Tabsd_uw[i,"cold"] <- attrdl(data$Tabsd_uw,cb,data$deaths,coef=coefallTabsd_uw[i,],
                                      vcov=vcovallTabsd_uw[[i]],type="an",dir="forw",
                                      range=c(-100,quantile(data$Tabsd_uw, 0.1, na.rm=T)), cen=mmt)
  
  matsim_Tabsd_uw[i,"m25"] <- attrdl(data$Tabsd_uw,cb,data$deaths,coef=coefallTabsd_uw[i,],
                                     vcov=vcovallTabsd_uw[[i]],type="an",dir="forw",
                                     range=c(-100,quantile(data$Tabsd_uw, 0.25, na.rm=T)), cen=mmt)
  
  matsim_Tabsd_uw[i,"total"] <- attrdl(data$Tabsd_uw,cb,data$deaths,coef=coefallTabsd_uw[i,],
                                       vcov=vcovallTabsd_uw[[i]],type="an",dir="forw", cen=mmt)
  
  # COMPUTE EMPIRICAL OCCURRENCES OF THE ATTRIBUTABLE DEATHS
  # USED TO DERIVE CONFIDENCE INTERVALS
  arraysim_Tabsd_uw[i,"heat",] <- attrdl(data$Tabsd_uw,cb,data$deaths,coef=coefallTabsd_uw[i,],
                                         vcov=vcovallTabsd_uw[[i]],type="an",dir="forw", sim=T,nsim=nsim,
                                         range=c(quantile(data$Tabsd_uw, 0.9, na.rm=T),100), cen=mmt)
  
  arraysim_Tabsd_uw[i,"m75",] <- attrdl(data$Tabsd_uw,cb,data$deaths,coef=coefallTabsd_uw[i,],
                                        vcov=vcovallTabsd_uw[[i]],type="an",dir="forw", sim=T,nsim=nsim,
                                        range=c(quantile(data$Tabsd_uw, 0.75, na.rm=T),100), cen=mmt)
  
  arraysim_Tabsd_uw[i,"cold",] <- attrdl(data$Tabsd_uw,cb,data$deaths,coef=coefallTabsd_uw[i,],
                                         vcov=vcovallTabsd_uw[[i]],type="an",dir="forw", sim=T,nsim=nsim,
                                         range=c(-100,quantile(data$Tabsd_uw, 0.1, na.rm=T)), cen=mmt)
  
  arraysim_Tabsd_uw[i,"m25",] <- attrdl(data$Tabsd_uw,cb,data$deaths,coef=coefallTabsd_uw[i,],
                                        vcov=vcovallTabsd_uw[[i]],type="an",dir="forw", sim=T,nsim=nsim,
                                        range=c(-100,quantile(data$Tabsd_uw, 0.25, na.rm=T)), cen=mmt)
  
  arraysim_Tabsd_uw[i,"total",] <- attrdl(data$Tabsd_uw,cb,data$deaths,coef=coefallTabsd_uw[i,],
                                          vcov=vcovallTabsd_uw[[i]],type="an",dir="forw", sim=T, nsim=nsim,cen=mmt)
  data <- fulldataset[[i]]
  
  #### - ERA5 ####
  # DEFINE THE CROSSBASIS
  argvar <- list(fun=varfun, degree=vardegree,
                 knots=quantile(data$ERA5,varper/100,na.rm=T), 
                 Bound=range(data$ERA5,na.rm=T))
  #Bound=range(tmeanper,na.rm=T))
  
  arglag <- list(fun="ns",knots=logknots(lag,lagnk))
  cb <- crossbasis(data$ERA5,lag=lag,argvar=argvar,arglag=arglag)
  
  # RUN THE MODEL AND OBTAIN PREDICTIONS
  model <- glm(formula,data,family=quasipoisson,na.action="na.exclude")
  
  # REDUCTION TO OVERALL CUMULATIVE
  pred <- crosspred(cb,model,cen=mean(data$ERA5,na.rm=T))
  redtemp <- crossreduce(cb,model,cen=mean(data$ERA5,na.rm=T))
  redtemp$predvar2 <- quantile(data$ERA5, 1:99/100)
  #mmt <-  redtemp$predvar2[40:99][which.min(redtemp$fit)[[1]]]
  mmt <- mmt_list[i,4] 
  
  redallERA5[[i]] <- crossreduce(cb,model,cen=mmt)
  plot(redallERA5[[i]])
  
  coefallERA5[i,] <- coef(redallERA5[[i]])
  vcovallERA5[[i]] <- vcov(redallERA5[[i]])
  qaicres[i,4] <- QAIC(model)
  
  cutheatERA5 <- quantile(data$ERA5,0.99, na.rm=T)
  cutcoldERA5 <- quantile(data$ERA5,0.01, na.rm=T)
  cutERA5 <- crossreduce(cb,model, at=c(cutheatERA5,cutcoldERA5), cen=mmt)
  
  tab_heat[i,4] <- paste0(formatC(cutERA5$RRfit[2],2, format="f"), sep=" (", 
                          formatC(cutERA5$RRlow[2],2, format="f"), sep=" - ",  
                          formatC(cutERA5$RRhigh[2],2, format="f"), sep=")")
  
  tab_cold[i,4] <- paste0(formatC(cutERA5$RRfit[1],2, format="f"), sep=" (", 
                          formatC(cutERA5$RRlow[1],2, format="f"), sep=" - ",  
                          formatC(cutERA5$RRhigh[1],2, format="f"), sep=")")
  
  tab_heat_ERA5[i,1:3] <- cbind(cutERA5$RRfit[2], cutERA5$RRlow[2],cutERA5$RRhigh[2])
  tab_cold_ERA5[i,1:3] <- cbind(cutERA5$RRfit[1], cutERA5$RRlow[1],cutERA5$RRhigh[1])
  
  # ESTIMATION OF THE ATTRIBUTABLE FRACTIONS
  matsim_era5[i,"heat"] <- attrdl(data$ERA5,cb,data$deaths,coef=coefallERA5[i,],
                                  vcov=vcovallERA5[[i]],type="an",dir="forw",
                                  range=c(quantile(data$ERA5, 0.9, na.rm=T),100), cen=mmt)
  
  matsim_era5[i,"m75"] <- attrdl(data$ERA5,cb,data$deaths,coef=coefallERA5[i,],
                                 vcov=vcovallERA5[[i]],type="an",dir="forw",
                                 range=c(quantile(data$ERA5, 0.75, na.rm=T),100), cen=mmt)
  
  matsim_era5[i,"cold"] <- attrdl(data$ERA5,cb,data$deaths,coef=coefallERA5[i,],
                                  vcov=vcovallERA5[[i]],type="an",dir="forw",
                                  range=c(-100,quantile(data$ERA5, 0.1, na.rm=T)), cen=mmt)
  
  matsim_era5[i,"m25"] <- attrdl(data$ERA5,cb,data$deaths,coef=coefallERA5[i,],
                                 vcov=vcovallERA5[[i]],type="an",dir="forw",
                                 range=c(-100,quantile(data$ERA5, 0.25, na.rm=T)), cen=mmt)
  
  matsim_era5[i,"total"] <- attrdl(data$ERA5,cb,data$deaths,coef=coefallERA5[i,],
                                   vcov=vcovallERA5[[i]],type="an",dir="forw", cen=mmt)
  
  # COMPUTE EMPIRICAL OCCURRENCES OF THE ATTRIBUTABLE DEATHS
  # USED TO DERIVE CONFIDENCE INTERVALS
  arraysim_era5[i,"heat",] <- attrdl(data$ERA5,cb,data$deaths,coef=coefallERA5[i,],
                                     vcov=vcovallERA5[[i]],type="an",dir="forw", sim=T,nsim=nsim,
                                     range=c(quantile(data$ERA5, 0.9, na.rm=T),100), cen=mmt)
  
  arraysim_era5[i,"m75",] <- attrdl(data$ERA5,cb,data$deaths,coef=coefallERA5[i,],
                                    vcov=vcovallERA5[[i]],type="an",dir="forw", sim=T,nsim=nsim,
                                    range=c(quantile(data$ERA5, 0.75, na.rm=T),100), cen=mmt)
  
  arraysim_era5[i,"cold",] <- attrdl(data$ERA5,cb,data$deaths,coef=coefallERA5[i,],
                                     vcov=vcovallERA5[[i]],type="an",dir="forw", sim=T,nsim=nsim,
                                     range=c(-100,quantile(data$ERA5, 0.1, na.rm=T)), cen=mmt)
  
  arraysim_era5[i,"m25",] <- attrdl(data$ERA5,cb,data$deaths,coef=coefallERA5[i,],
                                    vcov=vcovallERA5[[i]],type="an",dir="forw", sim=T,nsim=nsim,
                                    range=c(-100,quantile(data$ERA5, 0.25, na.rm=T)), cen=mmt)
  
  arraysim_era5[i,"total",] <- attrdl(data$ERA5,cb,data$deaths,coef=coefallERA5[i,],
                                      vcov=vcovallERA5[[i]],type="an",dir="forw", sim=T, nsim=nsim,cen=mmt)
  
  
  
  #### - UNWEIGHTED ERA5 ####
  # DEFINE THE CROSSBASIS
  argvar <- list(fun=varfun, degree=vardegree,
                 knots=quantile(data$ERA5_uw,varper/100,na.rm=T), 
                 Bound=range(data$ERA5_uw,na.rm=T))
  #Bound=range(tmeanper,na.rm=T))
  
  arglag <- list(fun="ns",knots=logknots(lag,lagnk))
  cb <- crossbasis(data$ERA5_uw,lag=lag,argvar=argvar,arglag=arglag)
  
  # RUN THE MODEL AND OBTAIN PREDICTIONS
  model <- glm(formula,data,family=quasipoisson,na.action="na.exclude")
  
  # REDUCTION TO OVERALL CUMULATIVE
  pred <- crosspred(cb,model,cen=mean(data$ERA5_uw,na.rm=T))
  redtemp <- crossreduce(cb,model,cen=mean(data$ERA5_uw,na.rm=T))
  redtemp$predvar2 <- quantile(data$ERA5_uw, 1:99/100)
  #mmt <-  redtemp$predvar2[40:99][which.min(redtemp$fit)[[1]]]
  mmt <- mmt_list[i,5] 
  
  redallERA5_uw[[i]] <- crossreduce(cb,model,cen=mmt)
  plot(redallERA5_uw[[i]])
  
  coefallERA5_uw[i,] <- coef(redallERA5_uw[[i]])
  vcovallERA5_uw[[i]] <- vcov(redallERA5_uw[[i]])
  qaicres[i,5] <- QAIC(model)
  
  cutheatERA5_uw <- quantile(data$ERA5_uw,0.99, na.rm=T)
  cutcoldERA5_uw <- quantile(data$ERA5_uw,0.01, na.rm=T)
  cutERA5_uw <- crossreduce(cb,model, at=c(cutheatERA5_uw,cutcoldERA5_uw), cen=mmt)
  
  
  tab_heat[i,5] <- paste0(formatC(cutERA5_uw$RRfit[2],2, format="f"), sep=" (", 
                          formatC(cutERA5_uw$RRlow[2],2, format="f"), sep=" - ",  
                          formatC(cutERA5_uw$RRhigh[2],2, format="f"), sep=")")
  
  tab_cold[i,5] <- paste0(formatC(cutERA5_uw$RRfit[1],2, format="f"), sep=" (", 
                          formatC(cutERA5_uw$RRlow[1],2, format="f"), sep=" - ",  
                          formatC(cutERA5_uw$RRhigh[1],2, format="f"), sep=")")
  
  tab_heat_ERA5_uw[i,1:3] <- cbind(cutERA5_uw$RRfit[2], cutERA5_uw$RRlow[2],cutERA5_uw$RRhigh[2])
  tab_cold_ERA5_uw[i,1:3] <- cbind(cutERA5_uw$RRfit[1], cutERA5_uw$RRlow[1],cutERA5_uw$RRhigh[1])
  
  
  # ESTIMATION OF THE ATTRIBUTABLE FRACTIONS
  matsim_era5_uw[i,"heat"] <- attrdl(data$ERA5_uw,cb,data$deaths,coef=coefallERA5_uw[i,],
                                     vcov=vcovallERA5_uw[[i]],type="an",dir="forw",
                                     range=c(quantile(data$ERA5_uw, 0.9, na.rm=T),100), cen=mmt)
  
  matsim_era5_uw[i,"m75"] <- attrdl(data$ERA5_uw,cb,data$deaths,coef=coefallERA5_uw[i,],
                                    vcov=vcovallERA5_uw[[i]],type="an",dir="forw",
                                    range=c(quantile(data$ERA5_uw, 0.75, na.rm=T),100), cen=mmt)
  
  matsim_era5_uw[i,"cold"] <- attrdl(data$ERA5_uw,cb,data$deaths,coef=coefallERA5_uw[i,],
                                     vcov=vcovallERA5_uw[[i]],type="an",dir="forw",
                                     range=c(-100,quantile(data$ERA5_uw, 0.1, na.rm=T)), cen=mmt)
  
  matsim_era5_uw[i,"m25"] <- attrdl(data$ERA5_uw,cb,data$deaths,coef=coefallERA5_uw[i,],
                                    vcov=vcovallERA5_uw[[i]],type="an",dir="forw",
                                    range=c(-100,quantile(data$ERA5_uw, 0.25, na.rm=T)), cen=mmt)
  
  matsim_era5_uw[i,"total"] <- attrdl(data$ERA5_uw,cb,data$deaths,coef=coefallERA5_uw[i,],
                                      vcov=vcovallERA5_uw[[i]],type="an",dir="forw", cen=mmt)
  
  # COMPUTE EMPIRICAL OCCURRENCES OF THE ATTRIBUTABLE DEATHS
  # USED TO DERIVE CONFIDENCE INTERVALS
  arraysim_era5_uw[i,"heat",] <- attrdl(data$ERA5_uw,cb,data$deaths,coef=coefallERA5_uw[i,],
                                        vcov=vcovallERA5_uw[[i]],type="an",dir="forw", sim=T,nsim=nsim,
                                        range=c(quantile(data$ERA5_uw, 0.9, na.rm=T),100), cen=mmt)
  
  arraysim_era5_uw[i,"m75",] <- attrdl(data$ERA5_uw,cb,data$deaths,coef=coefallERA5_uw[i,],
                                       vcov=vcovallERA5_uw[[i]],type="an",dir="forw", sim=T,nsim=nsim,
                                       range=c(quantile(data$ERA5_uw, 0.75, na.rm=T),100), cen=mmt)
  
  
  arraysim_era5_uw[i,"cold",] <- attrdl(data$ERA5_uw,cb,data$deaths,coef=coefallERA5_uw[i,],
                                        vcov=vcovallERA5_uw[[i]],type="an",dir="forw", sim=T,nsim=nsim,
                                        range=c(-100,quantile(data$ERA5_uw, 0.1, na.rm=T)), cen=mmt)
  
  arraysim_era5_uw[i,"m25",] <- attrdl(data$ERA5_uw,cb,data$deaths,coef=coefallERA5_uw[i,],
                                       vcov=vcovallERA5_uw[[i]],type="an",dir="forw", sim=T,nsim=nsim,
                                       range=c(-100,quantile(data$ERA5_uw, 0.25, na.rm=T)), cen=mmt)
  
  arraysim_era5_uw[i,"total",] <- attrdl(data$ERA5_uw,cb,data$deaths,coef=coefallERA5_uw[i,],
                                         vcov=vcovallERA5_uw[[i]],type="an",dir="forw", sim=T, nsim=nsim,cen=mmt)
  
  totdeathall[i] <- sum(data$deaths,na.rm=T)
  
}



an_insitu <- apply(arraysim_insitu,c(1,2),quantile,0.5, na.rm=T)
an_insitulow <- apply(arraysim_insitu,c(1,2),quantile,0.025, na.rm=T)
an_insituhigh <- apply(arraysim_insitu,c(1,2),quantile,0.975, na.rm=T)
rownames(an_insitu) <- rownames(an_insitulow) <- rownames(an_insituhigh) <- namesreg

an_tabsd <- apply(arraysim_Tabsd,c(1,2),quantile,0.5, na.rm=T)
an_tabsdlow <- apply(arraysim_Tabsd,c(1,2),quantile,0.025, na.rm=T)
an_tabsdhigh <- apply(arraysim_Tabsd,c(1,2),quantile,0.975, na.rm=T)
rownames(an_tabsd) <- rownames(an_tabsdlow) <- rownames(an_tabsdhigh) <- namesreg

an_tabsd_uw <- apply(arraysim_Tabsd_uw,c(1,2),quantile,0.5, na.rm=T)
an_tabsdlow_uw <- apply(arraysim_Tabsd_uw,c(1,2),quantile,0.025, na.rm=T)
an_tabsdhigh_uw <- apply(arraysim_Tabsd_uw,c(1,2),quantile,0.975, na.rm=T)
rownames(an_tabsd_uw) <- rownames(an_tabsdlow_uw) <- rownames(an_tabsdhigh_uw) <- namesreg

an_era5 <- apply(arraysim_era5,c(1,2),quantile,0.5, na.rm=T)
an_era5low <- apply(arraysim_era5,c(1,2),quantile,0.025, na.rm=T)
an_era5high <- apply(arraysim_era5,c(1,2),quantile,0.975, na.rm=T)
rownames(an_era5) <- rownames(an_era5low) <- rownames(an_era5high) <- namesreg

an_era5_uw <- apply(arraysim_era5_uw,c(1,2),quantile,0.5, na.rm=T)
an_era5low_uw <- apply(arraysim_era5_uw,c(1,2),quantile,0.025, na.rm=T)
an_era5high_uw <- apply(arraysim_era5_uw,c(1,2),quantile,0.975, na.rm=T)
rownames(an_era5_uw) <- rownames(an_era5low_uw) <- rownames(an_era5high_uw) <- namesreg


# ATTRIBUTABLE FRACTIONS
af_insitu <- an_insitu/totdeathall*100
af_insitulow <- an_insitulow/totdeathall*100
af_insituhigh <- an_insituhigh/totdeathall*100

af_tabsd <- an_tabsd/totdeathall*100
af_tabsdlow <- an_tabsdlow/totdeathall*100
af_tabsdhigh <- an_tabsdhigh/totdeathall*100

af_tabsd_unw <- an_tabsd_uw/totdeathall*100
af_tabsdlow_unw <- an_tabsdlow_uw/totdeathall*100
af_tabsdhigh_unw <- an_tabsdhigh_uw/totdeathall*100

af_ERA5 <- an_era5/totdeathall*100
af_ERA5low <- an_era5low/totdeathall*100
af_ERA5high <- an_era5high/totdeathall*100

af_ERA5_unw <- an_era5_uw/totdeathall*100
af_ERA5low_unw <- an_era5low_uw/totdeathall*100
af_ERA5high_unw <- an_era5high_uw/totdeathall*100













